import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateCareerDto } from "./dto/create-career.dto";
import { UpdateCareerDto } from "./dto/update-career.dto";
import { Career } from "./entities/career.entity";

@EntityRepository(Career)
export class CareerRepository extends Repository<Career> {
    async createEntity(dto: CreateCareerDto): Promise<Career | Error> {
        try {
            const {
                jobType,
                title,
                location,
                description,
                salary,
                departmentId,
                departmentName,
                departmentNameUrlSlug
            } = dto;
            const entity = this.create({
                jobType,
                title,
                location,
                description,
                salary,
                departmentId,
                departmentName,
                departmentNameUrlSlug
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<Career | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateCareerDto): Promise<Career | Error> {
        try {
            await this.update(id, {
                jobType: dto.jobType,
                title: dto.title,
                location: dto.location,
                description: dto.description,
                salary: dto.salary,
                departmentId: dto.departmentId,
                departmentName: dto.departmentName,
                departmentNameUrlSlug: dto.departmentNameUrlSlug
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<any | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findAllWithPagination(
        query: any,
        page: number,
        limit: number
    ): Promise<any | Error> {
        try {
            const count = await this.find({
                where: {
                    ...query,
                    isDeleted: false
                }
            });

            const careers = await this.find({
                where: {
                    ...query,
                    isDeleted: false
                },
                take: limit,
                skip: limit * (page - 1)
            });
            const pageCount = Math.ceil(count.length / limit);

            return {
                page: page,
                limit: limit,
                pageCount: pageCount,
                careers
            };
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(Career)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async countByDepartmentName(name: string): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    departmentName: name,
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async totalCareerCount(): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async getLatestCareerList(): Promise<any | Error> {
        try {
            return await this.find({
                where: {
                    isDeleted: false
                },
                order: {
                    createdAt: "DESC"
                },
                take: 6
            });
        } catch (e) {
            return e;
        }
    }
}